// WriteRemoteFile.cpp: implementation of the CRemoteFile class.
//
//////////////////////////////////////////////////////////////////////

#include <windows.h>
#include <assert.h>
#include <fstream.h>

#include "WriteRemoteFile.h"


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CRemoteFile::CRemoteFile()
{
	m_lpszName = NULL;
	m_lpszRegKey = NULL;

	m_lpszInFile = NULL;
	m_lpszOutFile = NULL;

	m_hwndProgress = NULL;
	m_hwndMessages = NULL;

	m_utInFileType = umlautetyp_undefined;
	m_utOutFileType = umlautetyp_undefined;
}

CRemoteFile::~CRemoteFile()
{
	// Evtl. vorhandene Strings lschen
	if ( m_lpszName != NULL )
		delete [] m_lpszName;
	if ( m_lpszRegKey != NULL )
		delete [] m_lpszRegKey;

	if ( m_lpszInFile != NULL )
		delete [] m_lpszInFile;
	if ( m_lpszOutFile != NULL )
		delete [] m_lpszOutFile;
}



//////////////////////////////////////////////////////////////////////
// Funktionen zum Setzen der Werte
//////////////////////////////////////////////////////////////////////



//--------------------------------------------------------------------
// Funktion SetNameAndRegKey:
//        Setzen des Benutzernamens und Registrationsschlssels
//
// Parameter :
//        lpszName  = Benutzername
//        lpszRegKey= Registrationsschlssel
//
// Ergebnis :
//        true = Kein Fehler
//--------------------------------------------------------------------
bool CRemoteFile::SetNameAndRegKey(char * lpszName, char * lpszRegKey)
{
	// Evtl. bereits vorhandene Strings lschen
	if ( m_lpszName != NULL )
		delete [] m_lpszName;
	if ( m_lpszRegKey != NULL )
		delete [] m_lpszRegKey;

	// Strings bernehmen, bei NULL-Zeigern leere Strings:
	m_lpszName = new char[strlen(lpszName) + 1];
	if ( strlen(lpszName) == 0 )
		strcpy(m_lpszName, "");
	else
		strcpy(m_lpszName, lpszName);

	m_lpszRegKey = new char[strlen(lpszRegKey) + 1];
	if ( strlen(lpszRegKey) == 0 )
		strcpy(m_lpszRegKey, "");
	else
		strcpy(m_lpszRegKey, lpszRegKey);

	return true;
}


//--------------------------------------------------------------------
// Funktion SetFileNames:
//        Setzen der Dateinamen fr Ein- und Ausgabedatei
//
// Parameter :
//        lpszInFile = Pfad und Name der Eingabedatei
//                     (Darf kein Leerstring und nicht NULL sein)
//        lpszOutFile= Pfad und Name der Ausgabedatei
//                     (Darf kein Leerstring und nicht NULL sein)
//
// Ergebnis :
//        true = Kein Fehler
//        false= Fehler durch GetLastError() ermitteln. Es treten auf:
//               ERROR_INVALID_PARAMETER
//--------------------------------------------------------------------
bool CRemoteFile::SetFileNames(char * lpszInFile, char * lpszOutFile)
{
	// Strings auf Korrektheit berprfen
	if ( (lpszInFile == NULL) || (strlen(lpszInFile) == 0) ||
		 (lpszOutFile == NULL) || (strlen(lpszOutFile) == 0) )
	{
		SetLastError(ERROR_INVALID_PARAMETER);
		return false;
	}

	// Evtl. bereits vorhandene Strings lschen
	if ( m_lpszInFile != NULL )
		delete [] m_lpszInFile;
	if ( m_lpszOutFile != NULL )
		delete [] m_lpszOutFile;

	// Strings bernehmen
	m_lpszInFile = new char[strlen(lpszInFile) + 1];
	strcpy(m_lpszInFile, lpszInFile);

	m_lpszOutFile = new char[strlen(lpszOutFile) + 1];
	strcpy(m_lpszOutFile, lpszOutFile);

	return true;
}


//--------------------------------------------------------------------
// Funktion SetWindowHandles:
//        Setzen der WindowHandles fr Progress-Control und
//        Fenster zur Bearbeitung der Messages
//
// Parameter :
//        hwndProgress= Handle der Progress-Control
//                      NULL = Kein Progress-Control
//        hwndMessages= Handle des Fensters zur Message-Bearbeitung
//                      NULL = Es werden keine Messages bearbeitet
//
// Ergebnis :
//        true = Kein Fehler
//--------------------------------------------------------------------
bool CRemoteFile::SetWindowHandles(HWND hwndProgress, HWND hwndMessages)
{
	m_hwndProgress = hwndProgress;
	m_hwndMessages = hwndMessages;

	return true;
}


//--------------------------------------------------------------------
// Funktion SetFileTypes:
//        Setzen der Dateiformate von Ein- und Ausgabedatei
//
// Parameter :
//        utInFileType = Typ der Umlautedarstellung der Eingabedatei
//        utOutFileType= Typ der Umlautedarstellung der Ausgabedatei
//        utInFileType und utOutFileType drfen nicht gleich sein
//
// Ergebnis :
//        true = Kein Fehler
//        false= Fehler durch GetLastError() ermitteln. Es treten auf:
//               ERROR_INVALID_PARAMETER
//--------------------------------------------------------------------
bool CRemoteFile::SetFileTypes(umlautetyp utInFileType, umlautetyp utOutFileType)
{
	if ( utInFileType == utOutFileType )
	{
		SetLastError(ERROR_INVALID_PARAMETER);
		return false;
	}

	m_utInFileType = utInFileType;
	m_utOutFileType = utOutFileType;

	return true;
}



//////////////////////////////////////////////////////////////////////
// Funktionen zur Dateibehandlung
//////////////////////////////////////////////////////////////////////



//--------------------------------------------------------------------
// Funktion WriteRemoteFile:
//        Schreiben der Remote-Datei
//
// Parameter :
//        lpszFilename = Pfad und Name der Remote-Datei
//
// Ergebnis :
//        true = Kein Fehler
//        false= Fehler durch GetLastError() ermitteln. Es treten auf:
//               ERROR_OPEN_FAILED, ERROR_DISK_OPERATION_FAILED
//--------------------------------------------------------------------
bool CRemoteFile::WriteRemoteFile(const char * lpszFilename)
{
	ofstream	OutFile;
	DWORD		dwTemp;

	// Datei ffnen
	OutFile.open(lpszFilename, ios::out | ios::trunc | ios::binary);
	if ( !OutFile.good() )
	{
		SetLastError(ERROR_OPEN_FAILED);
		return false;
	}

	// Header schreiben :
	// Kennung fr UmlauteKonverter-RemoteFile
	OutFile.write("UmlK", 4);
	// Versionsnummer
	dwTemp = 1;
	OutFile.write((unsigned char*) &dwTemp, 4);

	// Registrationsdaten schreiben :
	// Benutzername inkl. abschlieendes Nullbyte :
	dwTemp = strlen(m_lpszName) + 1;
	OutFile.write((unsigned char*) &dwTemp, 4);
	OutFile.write(m_lpszName, dwTemp);
	// Registrationsschlssel inkl. abschlieendes Nullbyte :
	dwTemp = strlen(m_lpszRegKey) + 1;
	OutFile.write((unsigned char*) &dwTemp, 4);
	OutFile.write(m_lpszRegKey, dwTemp);

	// Angaben ber Eingabedatei schreiben :
	// Name der Eingabedatei inkl. abschlieendes Nullbyte :
	dwTemp = strlen(m_lpszInFile) + 1;
	OutFile.write((unsigned char*) &dwTemp, 4);
	OutFile.write(m_lpszInFile, dwTemp);
	// Format der Eingabedatei
	OutFile.write((unsigned char*) &m_utInFileType, 2);

	// Angaben ber Ausgabedatei schreiben :
	// Name der Ausgabedatei inkl. abschlieendes Nullbyte :
	dwTemp = strlen(m_lpszOutFile) + 1;
	OutFile.write((unsigned char*) &dwTemp, 4);
	OutFile.write(m_lpszOutFile, dwTemp);
	// Format der Ausgabedatei
	OutFile.write((unsigned char*) &m_utOutFileType, 2);

	// Window-Handles des aufrufenden Programmes schreiben :
	// Handle der Progress-Control
	assert(sizeof(m_hwndProgress) == 4);
	OutFile.write((unsigned char*) &m_hwndProgress, 4);
	// Handle des Fensters, das die Messages bearbeitet
	assert(sizeof(m_hwndMessages) == 4);
	OutFile.write((unsigned char*) &m_hwndMessages, 4);


	// Datei schlieen
	OutFile.close();
	if ( !OutFile.good() )
	{
		SetLastError(ERROR_DISK_OPERATION_FAILED);
		return false;
	}

	return true;
}



